#!/usr/bin/env python

############################################################################
# Desktop Script
#
# Luis Cidoncha <luisc@acm.asoc.fi.upm.es>
# Jorge Lopez <jorge.lopez@acm.org>
# (c) 2005 
#
# Depends on: Python 2.2, PyQt, PyKDE
#
# See the README file for more info
############################################################################
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
############################################################################


import sys
import math
import threading
import Queue
from os import *
from ConfigParser import *

try:
    from qt import *
except:
    popen( "kdialog --sorry 'PyQt (QT bindings for Python) is required for this script.'" )
    raise

try:
    from kdecore import *
except:
    popen( "kdialog --sorry 'PyKDE (KDE bindings for Python) is required for this script.'" )
    raise


debug_prefix = "[Desktop Script]"

class ConfigDialog(QDialog):

    def __init__(self,app):
        QDialog.__init__(self)
        self.setWFlags(Qt.WDestructiveClose)
        KWin.setState(self.winId(), NET.KeepAbove )
        net = NETRootInfo(qt_xdisplay())
        self.setCaption("Desktop Script - amaroK")

        self.ok = QPushButton(self)
        self.ok.setGeometry(QRect(30,190,81,31))
        self.cancel = QPushButton(self)
        self.cancel.setGeometry(QRect(140,190,81,31))
        self.filter = QCheckBox(self)
        self.filter.setGeometry(QRect(30,30,150,20))
        self.position = QGroupBox(self)
        self.position.setGeometry(QRect(30,70,190,100))
        self.x_label = QLabel(self.position)
        self.x_label.setGeometry(QRect(30,26,30,21))
        self.y_label = QLabel(self.position)
        self.y_label.setGeometry(QRect(30,60,30,20))
        self.x_box = QSpinBox(self.position)
        self.x_box.setGeometry(QRect(70,26,90,22))
        self.x_box.setMinValue(-1*app.width)
        self.x_box.setMaxValue(net.desktopGeometry(0).width)
        self.y_box = QSpinBox(self.position)
        self.y_box.setGeometry(QRect(70,60,90,22))
        self.y_box.setMinValue(-1*app.height)
        self.y_box.setMaxValue(net.desktopGeometry(0).height)
        self.__readSettings()

        self.ok.setText("Ok")
        self.cancel.setText("Cancel")
        self.filter.setText("Apply light filter")
        self.position.setTitle("Position on desktop")
        self.x_label.setText("X:")
        self.y_label.setText("Y:")

        self.connect( self.ok,     SIGNAL("clicked()"), self.__save )
        self.connect( self.cancel, SIGNAL("clicked()"), self, SLOT("reject()") )

        self.connect( self.x_box, SIGNAL("valueChanged(int)"), app.setX )
        self.connect( self.y_box, SIGNAL("valueChanged(int)"), app.setY )
        self.connect( self.filter, SIGNAL("toggled(bool)"), app.setFilter )

        self.adjustSize()

    def __del__(self):
        pass

    def __readSettings(self):
        config = ConfigParser()
        config.read("desktoprc")
        try:
#            light = config.getboolean("General","filter")
            light = config.get("General","filter")
            self.filter.setOn(light=="True")
            x = config.getint("Position","x")
            self.x_box.setValue(x)
            y = config.getint("Position","y")
            self.y_box.setValue(y)
        except:
            self.filter.setOn(True)
            self.x_box.setValue(50)
            self.y_box.setValue(50)

    def __save(self):
        self.file = file("desktoprc",'w')
        self.config = ConfigParser()
        self.config.add_section("General")
        self.config.add_section("Position")
        self.config.set("General","filter", self.filter.isOn())
        self.config.set("Position","x", self.x_box.value())
        self.config.set("Position","y", self.y_box.value())
        self.config.write(self.file)
        self.file.close()

        self.accept()

class DesktopD(QMainWindow):

    ver = "0.2"
    codename = "pa ti Montse, pa ti"

    def __init__(self):
        QMainWindow.__init__(self,None,None,Qt.WStyle_Customize |
                Qt.WRepaintNoErase  |
                Qt.WStyle_NoBorder  |
                Qt.WDestructiveClose )

        self.width = 205
        self.height = 178
        self.coverWidth = 176
        self.coverHeight = 171
        self.actual = ""
        self.x = 50
        self.y = 50
        self.filter = True
        self.isHide = True
        self.hasCover = False

        #KGlobal.dirs() give a segfault :(
        self.up = QImage("../scripts/desktop/img/up.png")
        self.bottom = QImage("../scripts/desktop/img/bottom.png")
        self.right = QImage("../scripts/desktop/img/right.png")

        KWin.setType(self.winId(), NET.Dock)
        self.setCaption("Desktop Script - amaroK")

        self.image = QLabel(self)
        self.image.setGeometry(QRect(0,0,self.width,self.height))

        self.paintImage(True)

        self.queue = Queue.Queue()
        self.startTimer(100)
        self.t = threading.Thread(target = self.__readStdin)
        self.t.start()

        self.setMaximumSize(self.width,self.height)
        self.setMinimumSize(self.width,self.height)
        self.resize(self.width,self.height)
        self.readSettings()
        self.__show()

    def __del__(self):
        pass

    def paintImage (self,force):
        song = popen("dcop amarok player coverImage").read()[0:-1]
        if QString(song).contains("nocover.png"):
            self.hasCover = False
            self.__hide()
        elif (self.actual != song or force):
            self.hasCover = True
            cover = self.__getCover(song)
            if self.filter:
                cover = self.__filter(cover)
            self.image.setPixmap(self.__box(QPixmap(cover)))
            self.__show()
        self.actual = song

    def __getCover(self, url):
        canvas = QPixmap(self.width,self.height)
        painter = QPainter(canvas)
        #cover
        painter.drawImage(23,4,QImage(url).smoothScale(self.coverWidth,self.coverHeight))
        #black strip
        painter.fillRect(3,3,17,self.height-6,QBrush(QColor(0,0,0)))

        return canvas

    def __filter(self, canvas):
        #you are not expected to understand this
        #TODO: fix the lame code or change it with a precalculated matrix
        cover = canvas.convertToImage()
        b_factor = 0.04
        j_factor = 0.2
        w = self.width
        h = self.height
        for i in range(0, w):
            for j in range(0, 150):
                pixel = cover.pixel(i,j)
                diff = math.sqrt(math.pow(i-170,2)+math.pow(j-160,2)) - 50 - j * j_factor
                if diff > 105:
                    t = ((diff*2.0) * b_factor * (((float(h)-j)/h) * 4) * (((float(w)-i)/w)) * 4)
                    if diff == 106 or diff == 107:
                        t = t/8
                    red = qRed(pixel) + t
                    green = qGreen(pixel) + t
                    blue = qBlue(pixel) + t
                    if red > 255:
                        red = 255
                    if green > 255:
                        green = 255
                    if blue > 255:
                        blue = 255
                    cover.setPixel(i,j,qRgb(red,green,blue))

        return cover

    def __box(self, cover):
        painter = QPainter(cover)
        #left
        painter.fillRect(0,4,1,self.height-8,QBrush(QColor(138,138,138)))
        painter.fillRect(1,4,1,self.height-8,QBrush(QColor(188,188,188)))
        painter.fillRect(2,4,1,self.height-8,QBrush(QColor(160,160,160)))
        #strip
        painter.fillRect(20,4,1,self.height-8,QBrush(QColor(138,138,138)))
        painter.fillRect(21,4,1,self.height-8,QBrush(QColor(188,188,188)))
        painter.fillRect(22,4,1,self.height-8,QBrush(QColor(130,130,130)))
        #up
        painter.drawImage(0,0,self.up)
        #bottom
        painter.drawImage(0,self.height-4,self.bottom)
        #right
        painter.drawImage(self.width-8,4,self.right)

        return cover

    def __readStdin(self):
        while True:
            line = sys.stdin.readline()

            if line:
                self.queue.put_nowait( line )
            else:
                break

    def timerEvent(self, event):
        if not self.queue.empty():
            string = QString(self.queue.get_nowait())
            debug( str( string ) )

            if string.contains("configure"):
                self.configure()
            elif string.contains("trackChange"):
                self.paintImage(False)
            elif string.contains("engineStateChange"):
                if (string.contains("playing") or string.contains("paused")):
                    self.__show()
                else:
                    self.__hide()

    def __hide(self):
        if not self.isHide:
            self.hide()
            self.isHide = True

    def __show(self):
        if self.hasCover and self.isHide:
            KWin.setState(self.winId(), NET.KeepBelow | NET.SkipPager | NET.SkipTaskbar)
            KWin.setOnAllDesktops(self.winId(), True)
            self.show()
            self.lowerW()
            self.isHide = False

    def readSettings(self):
        self.lowerW()
        config = ConfigParser()
        config.read("desktoprc")
        try:
#            light = config.getboolean("General","filter")
            self.filter = config.get("General","filter") == "True"
            self.x = config.getint("Position","x")
            self.y = config.getint("Position","y")
        except:
            self.filter = True
            self.x = 50
            self.y = 50
        self.move(self.x,self.y)
        self.paintImage(True)

    def configure(self):
        self.dia = ConfigDialog(self)
        self.dia.show()
        self.connect(self.dia, SIGNAL("destroyed()"), self.readSettings)

    def mousePressEvent(self, e):
        self.lowerW()
        popen("dcop amarok player showOSD")

    def setX(self, x):
        self.x = x
        self.move(self.x,self.y)

    def setY(self, y):
        self.y = y
        self.move(self.x,self.y)

    def setFilter(self, b):
        self.filter = b
        self.paintImage(True)

def debug( message ):
    """ Prints debug message to stdout """

    print debug_prefix + " " + message

if __name__ == "__main__":
    app = QApplication(sys.argv)
    DesktopD().lowerW()
    app.exec_loop()
